import os
import sys

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import time
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
import torch.utils.data as data
import torchvision.transforms as transforms
from torch.autograd import Variable

from collections import OrderedDict
from tqdm import tqdm
import numpy as np
import glob
import random
import os
import clip

from model.framework import STLN


from itertools import permutations

from ..logger import init_logger
from .utils import DataLoader as myDL
from .utils import setup_seed, np_load_frame


class novaArgs:
    def __init__(self, args):
        self.args = args
        self.seed = setup_seed(self.args.seed)
        self.logger = self.build_logger()
        

    def build_logger(self):
        return init_logger(self.args).logger
        
    
class novaData(novaArgs):
    def __init__(self, args):
        super().__init__(args)
        self.train_batch = self.build_train_batch()
        self.test_batch = self.build_test_batch()
        
    def build_train_batch(self):
        train_dataset = myDL(
            self.args,
            self.args.train_folder,
            transforms.Compose(
                [
                    transforms.ToTensor(),
                ]
            ),
            resize_height=self.args.h,
            resize_width=self.args.w,
            time_step=self.args.t_length - 1,
        )
        self.args.train_size = len(train_dataset)
        train_batch = data.DataLoader(
            train_dataset,
            batch_size=self.args.batch_size,
            shuffle=True,
            num_workers=self.args.num_workers,
            drop_last=True,
        )
        return train_batch

    def build_test_batch(self):
        test_dataset = DataLoader(
            self.args.test_folder,
            transforms.Compose(
                [
                    transforms.ToTensor(),
                ]
            ),
            resize_height=self.args.h,
            resize_width=self.args.w,
            time_step=self.args.t_length - 1,
        )

        self.args.test_size = len(test_dataset)

        test_batch = data.DataLoader(
            test_dataset,
            batch_size=self.args.test_batch_size,
            shuffle=False,
            num_workers=self.args.num_workers_test,
            drop_last=False,
        )
        return test_batch


class novaTrainer(novaData):
    def __init__(self, args):
        super().__init__(args)
        
        self.model = self.build_model()

        self.optimizer = self.build_optimizer()
        self.scheduler = self.build_scheduler()
        self.loss_func_mse = self.build_loss_fun()

        self.m_items = F.normalize(
            torch.rand((args.msize, args.mdim), dtype=torch.float), dim=1
        ).cuda()  # Initialize the memory items

    def build_model(self):
        return None

    def build_optimizer(self):
        return torch.optim.Adam(self.model.parameters(), lr=self.args.lr)
    
    def build_scheduler(self):
        return optim.lr_scheduler.CosineAnnealingLR(
            self.optimizer, T_max=self.args.epochs
        )

    def build_loss_fun(self):
        return nn.MSELoss(reduction="none")


class novaEvaluator(novaData):
    def __init__(self, args):
        super().__init__(args)

    def build_test_batch(self):
        test_dataset = myDL(
            self.args,
            self.args.test_folder,
            transforms.Compose(
                [
                    transforms.ToTensor(),
                ]
            ),
            resize_height=self.args.h,
            resize_width=self.args.w,
            time_step=self.args.t_length - 1,
        )

        self.args.test_size = len(test_dataset)

        test_batch = data.DataLoader(
            test_dataset,
            batch_size=self.args.test_batch_size,
            shuffle=False,
            num_workers=self.args.num_workers_test,
            drop_last=False,
        )
        return test_batch


class novaRunner(novaTrainer, novaEvaluator):
    def __init__(self, args):
        super().__init__(args)
        self.now_acc = 0
        self.best_acc = -1

    def _run_batch(self, inputs):
        images, clip_feas = inputs
        imgs, imgs_r, gt_r = self.inputs2imgs(images)
        clip_feas = clip_feas.cuda()
        
        # semantic_p  = self._clip(inputs[-2])
        # semantic_f = self._clip(inputs[-1])

        # context_p = semantic_p.view(-1, 512, 1, 1).expand(-1, 512, 32, 32)

        out_list, seq, pred_sem, non_sem = self.model.forward(
            imgs[:, 0 : 3 * (self.args.t_length - 1)],
            self.m_items,
            imgs_r,
            clip_feas[:, :4],
            True)

        (   outputs,
            _,
            _,
            m_items,
            softmax_score_query,
            softmax_score_memory,
            separateness_loss,
            compactness_loss,
        ) = out_list


        loss_pixel = torch.mean(
            self.loss_func_mse(outputs, imgs[:, 3 * (self.args.t_length - 1) :])
        )

        loss_r = self.compute_loss(seq, gt_r)
        loss_semantic = self.bak_compute_semantic_loss(pred_sem, clip_feas[:, -1])
        # loss_semantic = self.compute_semantic_loss(pred_sem, clip_feas[:, -1], non_sem)
        

        loss = (
            loss_pixel
            + self.args.loss_compact * compactness_loss
            + self.args.loss_separate * separateness_loss  
            + loss_r * self.args.loss_w_sorting
            + loss_semantic * self.args.loss_w_semantic
        )

        if self.iters is not None:
            self.logger.info(
                "iter: {} / Loss: Prediction {:.6f} / Compact: {:.6f} / Separate: {:.6f} / Sort: {:.6f} / Semantic: {:.6f}".format(
                    self.iters,
                    loss_pixel.item(),
                    separateness_loss.item(),
                    compactness_loss.item(),
                    loss_r.item(),
                    loss_semantic.item()
                )
            )

        self.optimizer.zero_grad()
        loss.backward(retain_graph=True)
        self.optimizer.step()

    def _run_batch_test(self, inputs):
        pass

    def _run_epoch(self):
        with tqdm(total=len(self.train_batch)) as pbar:
            pbar.set_description("Epoch {} / {}".format(self.epoch, self.args.epochs))
            for i, inputs in enumerate(self.train_batch):
                self.iters = i + self.epoch * len(self.train_batch) 
                # self.iters = i 

                self.iters = self.iters if self.iters % self.args.logger_iters == 0 else None

                self.model.train()
                self._run_batch(inputs)

                pbar.update(1)
            pbar.close()
        self.scheduler.step()

    def _save_checkpoint(self):
        self.logger.info("now: {}, pre_best: {}".format(self.now_acc, self.best_acc))
        if self.now_acc > self.best_acc:
            self.best_acc = self.now_acc
            self.logger.info("Epoch {}, Best checkpoint is saved at {}".format(self.epoch, self.args.output_dir))
            torch.save(self.model.state_dict(), os.path.join(self.args.output_dir, "model_best.pth"))
            torch.save(self.m_items, os.path.join(self.args.output_dir, "keys_best.pt"))
        else:
            torch.save(self.model.state_dict(), os.path.join(self.args.output_dir, "model.pth"))
            torch.save(self.m_items, os.path.join(self.args.output_dir, "keys.pt"))
            
            self.logger.info(f"Epoch {self.epoch} | Training checkpoint saved at {self.args.output_dir}")
            
    def train(self):
        self.model.train()
        
        self.logger.info("Training dataset {}~".format(self.args.dataset_type))
        self.logger.info("The output directory is {}".format(self.args.output_dir))
        
        if self.args.load_best_dir != '':
            print('*'*30)
            print('loading pre-trained weights:', self.args.load_best_dir)
            print('*'*30)
    
            model_path = os.path.join(self.args.load_best_dir, 'model_best.pth')
            m_items_path = os.path.join(self.args.load_best_dir, 'keys_best.pt')
            
            state_dict = torch.load(model_path)
            # model = STLN(self.args).cuda()
            self.model.load_state_dict(state_dict, strict=False)
            self.m_items = torch.load(m_items_path).cuda().clone()

        for epoch in range(self.args.epochs):
            self.epoch = epoch
            self.logger.info(
                "Epoch: {} & Totol epochs: {}".format(self.epoch, self.args.epochs)
            )

            self.model.train()
            self._run_epoch()

            if ((self.epoch + 1) % self.args.eval_epochs == 0) or (
                self.epoch == self.args.epochs - 1
            ):
                # self.evaluate()
                self.test()
                self._save_checkpoint()
                

        self.logger.info("Training is finished")



import os
import re
from matplotlib import pyplot as plt
import seaborn as sns
import numpy as np

from pandas.core.frame import DataFrame
# plt.style.use('ggplot')

sns.set_style("whitegrid")

# log_path = "/userHome/nemo/outputs/ttaa1010_time/2023_10_11_21_54_12.log"
# log_path = "/userHome/nemo/outputs/ttaa1010_time/2023_10_18_00_04_55.log"
log_path = "/userHome/nemo/outputs/ttaa1027a_time/2023_10_27_20_48_44.log"

pattern = re.compile(r'Epoch:\ \d+\n')

patt1 = re.compile(r'Loss:\ Prediction\ \d+\.?\d*/\ Compactness\ \d+\.?\d*/\ Separateness\ \d+\.?\d*/\ ranking_loss\ \d+\.?\d*\n')
patt2 = re.compile(r'AUC:\ \d+\.?\d*%\n')

num_patt = re.compile(r'\d+\.?\d*')

with open(log_path, encoding='utf-8') as file_obj:
    contents = file_obj.read()
    file_obj.close()

auc_list = [num_patt.findall(x) for x in patt2.findall(contents)]
auc_list = np.array([float(x[0]) for x in auc_list])[:-1]
epochs = np.array([x for x in range(len(auc_list))])

matches = patt1.findall(contents)
losses_list = [num_patt.findall(x) for x in matches]
losses_list = np.array(losses_list, dtype=np.float32).T

print(len(epochs), len(auc_list), len(losses_list[0]), len(losses_list[1]), len(losses_list[2]), len(losses_list[3]))
data = DataFrame({'epoch': epochs, 'auc': auc_list, 'prediction': losses_list[0], 'compactness': losses_list[1], 'separateness': losses_list[2], 'ranking_loss': losses_list[3]})

fig, ax = plt.subplots(figsize=(12, 6))

# 
auc_max = np.max(data['auc'])
index_max = np.argmax(data['auc'])
print(auc_max, index_max)
xy = (index_max, auc_max)
plt.plot(index_max, auc_max, marker='v', markersize=5, c='r', zorder=2)
plt.annotate("(%s: %.2f%%)" % xy, xy=xy, xytext=(-20, 10), textcoords='offset points', 
    bbox=dict(boxstyle='round,pad=0.5', fc='yellow', ec='k', lw=1, alpha=0.5), zorder=3)

sns.lineplot(x='epoch', y='auc', c='darkorange', data=data, zorder=1)
plt.axvline(60, c="b", ls="-", lw=0.5, alpha=0.5)
plt.axhline(auc_list[0], c="b", ls=(45,(55,20)), lw=0.5)

p1 = ax.twinx()
p3 = ax.twinx()

sns.lineplot(x='epoch', y='prediction', data=data, ax=p1, color='b', ls='-.', lw=1, alpha=0.5, label='prediction')

sns.lineplot(x='epoch', y='compactness', data=data, ax=p1, color='g', ls='dashed', alpha=0.5, label='compactness')

sns.lineplot(x='epoch', y='separateness', data=data, ax=p3, color='m', ls='dotted', alpha=0.5, label='separateness')
sns.lineplot(x='epoch', y='ranking_loss', data=data, ax=p1, color='k', ls='dotted', alpha=0.5, label='ranking_loss')

p3.spines['right'].set_position(('outward', 80))

plt.subplots_adjust(left=0.05, right=0.8)
# plt.legend(loc='center left', bbox_to_anchor=(1.0, 0.5))
plt.legend(loc='center right', bbox_to_anchor=(1.0, 0.4))
p1.legend(loc='center right')

sns.despine()


plt.savefig('/userHome/nemo/outputs/ttaa1027a_time/2023_10_27_20_48_44.png')



import os
import cv2

def makeVideo(path, fps=10, size = (640, 360)):
    filelist = os.listdir(path)
    filelist2 = sorted([os.path.join(path, i) for i in filelist])
    
    video_path = os.path.join('/Users/tiumo/Desktop/SETS/tempcode', 'Video_{}.avi'.format(path.split('/')[-1]))

    video = cv2.VideoWriter(video_path, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'), fps,
                            size) 

    for item in filelist2:
        print(item)
        if item.endswith('._'):
            continue
        if item.endswith('.jpg'):
            print(item)
            img = cv2.imread(item)
            video.write(img)

    video.release()
    cv2.destroyAllWindows()
    print('视频合成生成完成啦')


if __name__ == '__main__':
    data_path = r'/Volumes/T7/#datasets/VAD-datasets/avenue/testing/frames/'
    frames_paths = os.listdir(data_path)
    for p in frames_paths:
        if '._' in p:
            continue
        path = os.path.join(data_path, p)
        makeVideo(path, 10)



import numpy as np
import os
import sys
import torch
import torch.nn as nn
import torch.nn.functional as F
import torchvision
import torchvision.utils as v_utils
import matplotlib.pyplot as plt
import cv2
import math
from collections import OrderedDict
import copy
import time
from sklearn.metrics import roc_auc_score


def rmse(predictions, targets):
    return np.sqrt(((predictions - targets) ** 2).mean())

def psnr(mse):

    return 10 * math.log10(1 / mse)

def get_lr(optimizer):
    for param_group in optimizer.param_groups:
        return param_group['lr']


def normalize_img(img):

    img_re = copy.copy(img)
    
    img_re = (img_re - np.min(img_re)) / (np.max(img_re) - np.min(img_re))
    
    return img_re

def point_score(outputs, imgs):
    
    loss_func_mse = nn.MSELoss(reduction='none')
    error = loss_func_mse((outputs[0]+1)/2,(imgs[0]+1)/2)
    normal = (1-torch.exp(-error))
    score = (torch.sum(normal*loss_func_mse((outputs[0]+1)/2,(imgs[0]+1)/2)) / torch.sum(normal)).item()
    return score
    
def anomaly_score(psnr, max_psnr, min_psnr):
    return ((psnr - min_psnr) / (max_psnr-min_psnr))

def anomaly_score_inv(psnr, max_psnr, min_psnr):
    return (1.0 - ((psnr - min_psnr) / (max_psnr-min_psnr)))

def anomaly_score_list(psnr_list):
    anomaly_score_list = list()
    for i in range(len(psnr_list)):
        anomaly_score_list.append(anomaly_score(psnr_list[i], np.max(psnr_list), np.min(psnr_list)))
        
    return anomaly_score_list

def anomaly_score_list_inv(psnr_list):
    anomaly_score_list = list()
    for i in range(len(psnr_list)):
        anomaly_score_list.append(anomaly_score_inv(psnr_list[i], np.max(psnr_list), np.min(psnr_list)))
        
    return anomaly_score_list

def AUC(anomal_scores, labels):
    frame_auc = roc_auc_score(y_true=np.squeeze(labels, axis=0), y_score=np.squeeze(anomal_scores))
    return frame_auc

def score_sum(list1, list2, alpha):
    list_result = []
    for i in range(len(list1)):
        list_result.append((alpha*list1[i]+(1-alpha)*list2[i]))
        
    return list_result



import os
import logging
import time


class htLogger:
    def __init__(self, args):
        # path init
        if "/auto" in args.output_dir:
            file_name = os.path.basename(args.config_file)[:-5]  # 使用配置文件名称
            args.output_dir = args.output_dir.replace("/auto", "/{}".format(file_name))
            if not os.path.exists(args.output_dir):
                os.makedirs(args.output_dir)

        # logger init
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.DEBUG)

        sh = logging.StreamHandler()

        # 创建处理器：sh为控制台处理器，fh为文件处理器,log_file为日志存放的文件夹
        if "_time" or "_test" in args.output_dir:
            log_file = os.path.join(
                args.output_dir,
                "{}.log".format(time.strftime("%Y_%m_%d_%H_%M_%S", time.localtime())),
            )
        else:
            log_file = os.path.join(args.output_dir, "eval.log")
        fh = logging.FileHandler(log_file, encoding="UTF-8")

        formator = logging.Formatter(
            fmt="%(asctime)s %(levelname)s: %(message)s",  # "%(asctime)s %(filename)s %(levelname)s %(message)s"
            datefmt="%m/%d-%Y %H:%M:%S",
        )
        sh.setFormatter(formator)
        fh.setFormatter(formator)

        self.logger.addHandler(sh)
        self.logger.addHandler(fh)

        self.logger.info(args)
        self.logger.info("-" * 100)
        
def init_logger(args):
    return htLogger(args).logger



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.offsetbox import OffsetImage, AnnotationBbox
from matplotlib.patches import FancyArrowPatch, Rectangle
from glob import glob
import os
from collections import OrderedDict
from PIL import Image, ImageOps
import cv2

from tqdm import tqdm

def load_test_data(label_dir, dataset_type, data_folder):
    labels = np.load(
        os.path.join(label_dir,
            "frame_labels_" + dataset_type + ".npy",
        )
    ).squeeze()
    
    
    videos = OrderedDict()
    test_folder = os.path.join(data_folder, dataset_type, "testing/frames")
    videos_list = sorted(glob(os.path.join(test_folder, "*")))

    label_length = 0
    
    for video in videos_list:
        video_name = video.split("/")[-1]
        videos[video_name] = {}
        videos[video_name]["path"] = video
        videos[video_name]["frame"] = glob(os.path.join(video, "*.jpg"))
        videos[video_name]["frame"].sort()
        videos[video_name]["length"] = len(videos[video_name]["frame"])
        
        videos[video_name]["label"] = labels[label_length: videos[video_name]["length"] + label_length]
                
        label_length += videos[video_name]["length"] 
    
    return labels, videos, videos_list

def add_semi_transparent_border(image, border_size=40, transparency=0.2):

    w, h = (image.shape[1], image.shape[0])
    
    # 在图像上画一个半透明的矩形
    cv2.rectangle(image, (border_size//2, border_size//2), (w-border_size//2, h-border_size//2), (0, 0, 255, int(255 * transparency)), border_size)

    return image

def open_image(image_path):
    image = np.array(cv2.imread(image_path))
     # 创建一个具有透明度的图像，使用BGRA格式
    image_with_alpha = np.zeros((image.shape[0], image.shape[1], 4), dtype=np.uint8)
    image_with_alpha[:, :, :3] = image  # 复制BGR通道
    image_with_alpha[:, :, 3] = 255  # 设置Alpha通道为255（不透明）
    return image_with_alpha


def img_movies(img):
    movies = cv2.imread("/userHome/nemo/projects/sorting/visualize/movie1.png", cv2.IMREAD_UNCHANGED)
    
    h1, w1 = img.shape[:2]
    h2, w2 = movies.shape[:2]
    
    movies = cv2.resize(movies, (w1, h2*w1//w2))
    img_array = np.concatenate((movies, img, movies), axis=0)  # 纵向拼接
    return img_array
    

def concat_images(frames, i_list, abnormals):
    for i, idx in enumerate(i_list):
        v = frames[idx]
        if i == 0:
            img_array = open_image(v)  # 打开图片
            img_array = img_movies(img_array)
            
            if idx in abnormals:
                # img = ImageOps.expand(img, border=n, fill='red')
                # img = img.resize((x, y))
                img_array= add_semi_transparent_border(np.array(img_array), border_size=20, transparency=0.5)
        else:
            img_array2 = open_image(v)
            img_array2 = img_movies(img_array2)
            if idx in abnormals:
                img_array2 = add_semi_transparent_border(np.array(img_array2), border_size=20, transparency=0.5)

            img_array = np.concatenate((img_array, img_array2), axis=1)  # 横向拼接
            # img_array = np.concatenate((img_array, img_array2), axis=0)  # 纵向拼接
    return Image.fromarray(cv2.cvtColor(img_array, cv2.COLOR_BGR2RGB))


if __name__ == "__main__":
    labels, videos, videos_list = load_test_data(
        "/userHome/nemo/projects/sorting/data",
        "avenue",
        "/userHome/nemo/datasets/",
    )

    for video in tqdm(videos_list):
        video_name = video.split("/")[-1]
        ax = plt.figure()
        y = videos[video_name]["label"]
        x = range(0, len(y))
        
        abnormals = np.where(y == 1)[0]
        
        start_a = np.where(np.diff(y) == 1)[0]+1
        end_a = np.where(np.diff(y) == -1)[0]+1

        images = concat_images(videos[video_name]["frame"], [i for i in range(0, len(y), len(y)//6)], abnormals)
        # 然后【采样】标识异常帧
            
        ## 画图
        
        fig = plt.figure(figsize=(15,7))
        
        ax1 = plt.subplot(211)
        
        plt.imshow(images)
                
        plt.xticks([])  # 去掉x轴
        plt.yticks([])  # 去掉y轴
        plt.axis('off')  # 去掉坐标轴
        
        ax2 = plt.subplot(212)
        plt.plot(x, y)
        for s,e in zip(start_a, end_a):
            plt.axvspan(s, e, alpha=0.1, color='red')
            
            img_index = np.random.choice(range(s,e))
            random_number = np.random.uniform(0.4, 0.9)

            img = Image.open(videos[video_name]["frame"][img_index])

            imagebox =OffsetImage(img, zoom=0.15)
            ab = AnnotationBbox(imagebox, (img_index, random_number), frameon=False, pad=0)
            ax2.add_artist(ab) 
            
            arrow = FancyArrowPatch((img_index, 1), (img_index, random_number), arrowstyle='<-', mutation_scale=15, color='gray')
            ax2.add_patch(arrow)   
            
        plt.xlim(0, len(y))
        x_tricks = np.sort(np.concatenate([np.insert(start_a, 0, 0), np.append(end_a, len(y))]))
        plt.xticks(x_tricks, rotation=45)
        # plt.ylim(-0.1, 1)
        
        fig.tight_layout()
        fig.savefig('/userHome/nemo/projects/sorting/visualize/figs/label_{}.png'.format(video_name))
        plt.close()
        plt.close()
        


import torch
import numpy as np
import random

from collections import OrderedDict
import os
import glob
import cv2
import torch.utils.data as data


def setup_seed(seed):
    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)
    np.random.seed(seed)
    random.seed(seed)
    # torch.backends.cudnn.deterministic = True # 加了会变慢
    return seed


def np_load_frame(filename, resize_height, resize_width):
    """
    Load image path and convert it to numpy.ndarray. Notes that the color channels are BGR and the color space
    is normalized from [0, 255] to [-1, 1].

    :param filename: the full path of image
    :param resize_height: resized height
    :param resize_width: resized width
    :return: numpy.ndarray
    """
    image_decoded = cv2.imread(filename)
    image_resized = cv2.resize(image_decoded, (resize_width, resize_height))
    image_resized = image_resized.astype(dtype=np.float32)
    image_resized = (image_resized / 127.5) - 1.0
    return image_resized


class DataLoader(data.Dataset):
    def __init__(
        self,
        args,
        video_folder,
        transform,
        resize_height,
        resize_width,
        time_step=4,
        num_pred=1,
        is_concat=False,
    ):
        self.args = args
        self.dir = video_folder  # video_folder
        self.transform = transform
        self.videos = OrderedDict()
        self._resize_height = resize_height
        self._resize_width = resize_width
        self._time_step = time_step
        self._num_pred = num_pred
        self.setup()
        self.samples = self.get_all_samples()
        self.is_concat = is_concat

    def setup(self):
        videos = glob.glob(os.path.join(self.dir, "*"))
        videos2 = glob.glob(
            os.path.join(self.dir.replace("datasets", "datasets2"), "*")
        )

        # videos.extend(videos2)
        clip_path = self.dir.replace(self.args.dataset_path, self.args.clip_feas_path)

        # print('dir:', self.dir)
        # print(self.args.dataset_path, self.args.clip_feas_path)
        # print(clip_path)
        # aaaa

        for video in sorted(videos):
            video_name = video.split("/")[-1]
            self.videos[video_name] = {}
            self.videos[video_name]["path"] = video
            self.videos[video_name]["frame"] = glob.glob(os.path.join(video, "*.jpg"))
            self.videos[video_name]["frame"].sort()
            self.videos[video_name]["length"] = len(self.videos[video_name]["frame"])

            self.videos[video_name]["clip"] = np.load(
                os.path.join(clip_path, "feas_{}.npy".format(video_name))
            )

    def get_all_samples(self):
        frames = []
        videos = glob.glob(os.path.join(self.dir, "*"))
        for video in sorted(videos):
            video_name = video.split("/")[-1]
            for i in range(len(self.videos[video_name]["frame"]) - self._time_step):
                frames.append(self.videos[video_name]["frame"][i])

        return frames

    def __getitem__(self, index):
        video_name = self.samples[index].split("/")[-2]
        frame_name = int(self.samples[index].split("/")[-1].split(".")[-2])

        batch = []
        feas_batch = []
        for i in range(self._time_step + self._num_pred):
            image = np_load_frame(
                self.videos[video_name]["frame"][frame_name + i],
                self._resize_height,
                self._resize_width,
            )

            feas = self.videos[video_name]["clip"][frame_name + i]
            feas_batch.append(feas)

            if self.transform is not None:
                batch.append(self.transform(image))
        if self.is_concat:
            return [np.concatenate(batch, axis=0), np.concatenate(feas_batch, axis=0)]
        else:
            return [np.array(batch), np.array(feas_batch, dtype=np.float32)]

    def __len__(self):
        return len(self.samples)



import os
import yaml
import argparse
import sys


def merge_data(data_1, data_2):
    """
    使用 data_2 和 data_1 合成一个新的字典。
    对于 data_2 和 data_1 都有的 key, 如果data_2为None, 则用data_1, 否则用data_2。
    :param data_1:
    :param data_2:
    :return:
    """
    if isinstance(data_1, dict) and isinstance(data_2, dict):
        new_dict = {}
        d2_keys = list(data_2.keys())
        for d1k in data_1.keys():
            if d1k in d2_keys:  # d1,d2都有。去往深层比对
                d2_keys.remove(d1k)
                new_dict[d1k] = merge_data(data_1.get(d1k), data_2.get(d1k))
            else:
                new_dict[d1k] = data_1.get(d1k)  # d1有d2没有的key
        for d2k in d2_keys:  # d2有d1没有的key
            new_dict[d2k] = data_2.get(d2k)
        return new_dict
    else:
        if data_2 == None: #d2为空使用d1
            return data_1
        else:              #d2不为空使用d2
            return data_2

def loadyaml(file_path):
    with open(file_path, mode="r", encoding="utf-8") as f:
        yamlConf = yaml.load(f.read(), Loader=yaml.FullLoader)
    if "__BASE__" in yamlConf.keys():
        (filedir, _) = os.path.split(file_path) 
        temp_list = yamlConf["__BASE__"] if type(yamlConf["__BASE__"]) is list else [yamlConf["__BASE__"]] 
        for base_config in temp_list:
            yamlConf = merge_data(loadyaml(os.path.join(filedir, base_config)), yamlConf)
    return yamlConf


class Struct:   # 未用到  
    def __init__(self, **entries): 
        self.__dict__.update(entries)

def yaml2object(file_path):   # 未用到   
    yamlConf = loadyaml(file_path)
    args = Struct(**yamlConf)
    return args

def get_parser():
    parser = argparse.ArgumentParser(description = 'A method to update parser parameters using yaml files') 
    
    parser.add_argument("-c", "--config-file", metavar="FILE", help="path to config file")
    parser.add_argument('--gpus', nargs='+', type=str, help='gpus')
    
    parser.add_argument(
        "--resume",
        action="store_true",
        help="Whether to attempt to resume from the checkpoint directory. "
        "See documentation of `DefaultTrainer.resume_or_load()` for what it means.",
    )
    parser.add_argument("--eval-only", action="store_true", help="perform evaluation only")
    parser.add_argument("--num-gpus", type=int, default=1, help="number of gpus *per machine*")
    parser.add_argument("--num-machines", type=int, default=1, help="total number of machines")
    
    return parser 

class Dict(dict):
    __setattr__ = dict.__setitem__
    __getattr__ = dict.__getitem__
    
def dictToObj(dictObj):
    if not isinstance(dictObj, dict):
        return dictObj
    d = Dict()
    for k, v in dictObj.items():
        d[k] = dictToObj(v)
    return d

def get_args(config_file):
    default_args = get_parser().parse_args()
    default_args.config_file = config_file
    args = merge_data(loadyaml(config_file), vars(default_args))
    
    return dictToObj(args)

if __name__ == "__main__":
    args = get_args('/userHome/nemo/projects/MNAD_jigsaw/configs/rank.yaml')

    print(args.gpus, type(args.gpus))   